searchData={"items":[{"type":"module","title":"Pyro","doc":"`Pyro` is a component tooling library for `Phoenix`.\n\nTo learn more, check out the [About](about.html) page.\n\nTo install, follow the [Get Started](get-started.html) guide.","ref":"Pyro.html"},{"type":"module","title":"Pyro.Overrides","doc":"The overrides system allows defining component defaults while also enabling deep customization and ad-hoc overrides of Pyro component props. This allows consumers of your component library to trivially and granularly modify your presets.\n\nA `Pyro.Component` flags attrs with `overridable`, then leverages [`assign_overridables/1`](`Pyro.Component.assign_overridables/1`) to reference overrides set in these presets/custom override modules and load them as defaults.\n\nConfigure your app with:\n\n```\nconfig :pyro, :overrides,\n  [MyApp.Overrides]\n```\n\nThen, define your overrides in your custom module:\n\n```\ndefmodule MyApp.Overrides do\n  @moduledoc false\n  use Pyro.Overrides\n\n  override MyAppWeb.CoreComponents, :back do\n    set :class, \"text-lg font-black\"\n    set :icon_kind, :outline\n    set :icon_name, :arrow_left\n  end\nend\n```\n\nThe overrides will be merged left-to-right, returning the value in the *first* module that sets a given key. So in the above example, the ` ` component will have an `icon_name` default of `:arrow_left`, since the `MyApp.Overrides` was the first module in the list to provide that key:\n\n- You only need to define what you want to override from the other defaults\n- You can use any number of `:overrides` modules, though it is probably best to only use only 1-3 to keep things simple/efficient\n- If no modules define the value, it will simply be `nil`\n- If [`assign_overridables/1`](`Pyro.Component.assign_overridables/1`) is called on the component with the `required: true` attr option, an error will be raised if no configured overrides define a default","ref":"Pyro.Overrides.html"},{"type":"function","title":"Pyro.Overrides.configured_overrides/0","doc":"Get the configured override modules.","ref":"Pyro.Overrides.html#configured_overrides/0"},{"type":"macro","title":"Pyro.Overrides.override/3","doc":"Define overrides for a specific component.\n\nYou need to specify the module and function name of the component to override.","ref":"Pyro.Overrides.html#override/3"},{"type":"macro","title":"Examples - Pyro.Overrides.override/3","doc":"override Core, :back do\n      set :class, \"text-lg font-black\"\n    end","ref":"Pyro.Overrides.html#override/3-examples"},{"type":"function","title":"Pyro.Overrides.override_for/3","doc":"Get an override value for a given component prop.","ref":"Pyro.Overrides.html#override_for/3"},{"type":"macro","title":"Pyro.Overrides.set/2","doc":"Override a setting within a component.\n\nValue can be:\n\n* A literal value matching the type of the prop being overridden\n* A function capture of arity 1 with the argument `passed_assigns`, which is executed at runtime and is passed the component's `assigns`\n* Any other function capture, which will simply be passed along as a literal\n\nThe `passed_assigns` function capture allows for complex conditionals.\n\n> ##","ref":"Pyro.Overrides.html#set/2"},{"type":"macro","title":"Tip: {: .info} - Pyro.Overrides.set/2","doc":">\n> Be sure to include the module in the function capture, since this is a macro and will lose the reference otherwise.","ref":"Pyro.Overrides.html#set/2-tip-info"},{"type":"macro","title":"Examples - Pyro.Overrides.set/2","doc":"set :class, \"text-lg font-black\"\n    set :class, &__MODULE__.back_class/1\n    # ...\n    def back_class(passed_assigns) do","ref":"Pyro.Overrides.html#set/2-examples"},{"type":"module","title":"Pyro.Component","doc":"This is basically the same thing as `Phoenix.Component`, but Pyro extends the `attr/3` macro with:\n\n* `:css_classes` type\n* `:overridable` flag\n* `:values` supports an atom value (override key)\n\nPyro also provides `assign_overridables/1`, which automatically assigns all flagged `overridable` attrs with defaults from `Pyro.Overrides`","ref":"Pyro.Component.html"},{"type":"module","title":"Example - Pyro.Component","doc":"```elixir\ndefmodule MyApp.Components.ExternalLink do\n  @moduledoc \"\"\"\n  An external link component.\n  \"\"\"\n  use Pyro.Component\n\n  attr :overrides, :list, default: nil, doc: @overrides_attr_doc\n  attr :class, :css_classes, overridable: true, required: true\n  attr :href, :string, required: true\n  attr :rest, :global, include: ~w[download hreflang referrerpolicy rel target type]\n  slot :inner_block, required: true\n\n  def external_link(assigns) do\n    assigns = assign_overridables(assigns)\n    ~H\"\"\"\n     \n      <%= render_slot(@inner_block) %>\n     \n    \"\"\"\n  end\nend\n```\n\n> ##","ref":"Pyro.Component.html#module-example"},{"type":"module","title":"Note: {: .info} - Pyro.Component","doc":">\n> Only additional features will be documented here. Please see the `Phoenix.Component` docs for the rest, as they will not be duplicated here.","ref":"Pyro.Component.html#module-note-info"},{"type":"macro","title":"Pyro.Component.assign_overridables/1","doc":"This macro automatically assigns all the overridable attrs, and handles merging classes for `:css_classes` type attrs.\n\nIt *must* be called once in any component that contains overridable attrs.","ref":"Pyro.Component.html#assign_overridables/1"},{"type":"macro","title":"Example - Pyro.Component.assign_overridables/1","doc":"```\ndef external_link(assigns) do\n    assigns = assign_overridables(assigns)\n```","ref":"Pyro.Component.html#assign_overridables/1-example"},{"type":"macro","title":"Pyro.Component.attr/3","doc":"There are only a few things added to `Phoenix.Component.attr/3` by Pyro:\n\n* `:css_classes` type\n  * merges overridable defaults with passed prop values via `Pyro.Component.CSS.classes/1` (or bring your own merge function)\n  * less bloated HTML\n* `:overridable` flag (marks attribute to be overridden by `Pyro.Overrides`)\n* `:values` supports an atom value (override key, loaded by `Pyro.Overrides`)\n\nThere are compile time checks to ensure the following, but of note:\n\n* Attrs flagged as `overridable` cannot have a `default` - That's what overrides are for! 🚀\n* If flagged as `overridable` and `required`, a runtime exception will be raised if no configured overrides provide a default\n* If any attrs are flagged as overridable\n  * The first attribute must be:\n\n  ```\n  attr :overrides, :list, default: nil, doc: @overrides_attr_doc\n  ```\n\n  * `assign_overridables/1` must be called\n\nEverything else is handled by `Phoenix.Component.attr/3`, so please consult those docs for the rest.","ref":"Pyro.Component.html#attr/3"},{"type":"function","title":"Pyro.Component.encode_flash/2","doc":"Encode a flash message as a JSON binary with extra metadata options. This is necessary because Phoenix only allows binary messages, but many flash messages would be vastly improved by bespoke presentation.\n\nThis allows you to override the defaults for:\n\n* `:title` - The title above the message\n* `:close` - Auto-close the flash after `:ttl`\n* `:ttl` - The time-to-live in milliseconds\n* `:icon_name` - Name of the icon displayed in the title\n* `:style_for_kind` - Override which kind of style this flash should have","ref":"Pyro.Component.html#encode_flash/2"},{"type":"function","title":"Examples - Pyro.Component.encode_flash/2","doc":"```elixir\nsocket\n|> put_flash(\n    \"success\",\n    encode_flash(\n      \"\"\"\n      This flash closes when it *wants to*.\n      And has a custom title and icon.\n      \"\"\",\n      title: \"TOTALLY CUSTOM\",\n      ttl: 6_000,\n      icon_name: \"hero-beaker\"\n    )\n  )\n```","ref":"Pyro.Component.html#encode_flash/2-examples"},{"type":"type","title":"Pyro.Component.encode_flash_opts/0","doc":"","ref":"Pyro.Component.html#t:encode_flash_opts/0"},{"type":"module","title":"Pyro.Component.CSS","doc":"Utilities for managing CSS.","ref":"Pyro.Component.CSS.html"},{"type":"function","title":"Pyro.Component.CSS.classes/1","doc":"Merges classes together, providing a few conveniences. It is similar to how `Surface` merges class prop types or `Tails` minus the Tailwind-aware smart merging. This is a key tool for conditional style and reusable components, allow for very granular style overrides.\n\n  - Includes keys of maps or keywords with a `true` value\n  - Flattens nested lists\n  - Removes duplicate/empty classes\n  - Truncate all previous classes with `{:truncate, \"new classes\"}`\n  - Remove specific previous classes with `{:remove, [\"mb-2\"]}`","ref":"Pyro.Component.CSS.html#classes/1"},{"type":"function","title":"Examples - Pyro.Component.CSS.classes/1","doc":"iex> classes(\"py-2      px-2\")\n    \"py-2 px-2\"\n\n    iex> classes([\"py-2\", [\" px-2\"]])\n    \"px-2 py-2\"\n\n    iex> classes([\"py-2\", [\"px-2\", \"ml-2\"], {:truncate, \"ml-1\"}, :\"mr-2\"])\n    \"mr-2 ml-1\"\n\n    iex> classes([\"py-2\", [\"px-2\", \"ml-2\"], {:remove, \"ml-2\"}, :\"mr-2\"])\n    \"mr-2 px-2 py-2\"\n\n    iex> classes([\"py-2\", [\"px-2\", \"ml-2\"], {:remove, [:\"px-2\", \"asdf\", \"ml-2\"]}, :\"mr-2\"])\n    \"mr-2 py-2\"\n\n> ##","ref":"Pyro.Component.CSS.html#classes/1-examples"},{"type":"function","title":"Note: {: .info} - Pyro.Component.CSS.classes/1","doc":">\n> Order is not preserved in the output. This doesn't matter in terms of class precedence, but it does improve merge performance slightly.","ref":"Pyro.Component.CSS.html#classes/1-note-info"},{"type":"function","title":"Bring Your Own Merger - Pyro.Component.CSS.classes/1","doc":"You can also override this function with your own by setting the config option, e.g:\n\n```\nconfig :pyro, :css_class_merger, {Tails, :classes, 1}\n# or\nconfig :pyro, :css_class_merger, &Tails.classes/1\n```","ref":"Pyro.Component.CSS.html#classes/1-bring-your-own-merger"},{"type":"module","title":"Pyro.Component.Helpers","doc":"A library of helpers for using/authoring Pyro components.","ref":"Pyro.Component.Helpers.html"},{"type":"function","title":"Pyro.Component.Helpers.all_timezones/0","doc":"","ref":"Pyro.Component.Helpers.html#all_timezones/0"},{"type":"function","title":"Pyro.Component.Helpers.default_timezone/0","doc":"Provides a configurable fallback timezone. Defaults to `\"Etc/UTC\"`.\n\n```elixir\n# runtime.exs\nconfig :pyro, default_timezone: \"America/Chicago\"\n```\n\n> ##","ref":"Pyro.Component.Helpers.html#default_timezone/0"},{"type":"function","title":"Note: {: .warning} - Pyro.Component.Helpers.default_timezone/0","doc":">\n> Requires a timezone database to be properly installed and configured.","ref":"Pyro.Component.Helpers.html#default_timezone/0-note-warning"},{"type":"function","title":"Pyro.Component.Helpers.format_datetime/3","doc":"Formats datetimes to localized format.","ref":"Pyro.Component.Helpers.html#format_datetime/3"},{"type":"function","title":"Examples - Pyro.Component.Helpers.format_datetime/3","doc":"iex> format_datetime(DateTime.new!(~D[2016-05-24], ~T[13:26:08.003], \"Etc/UTC\"), default_timezone())\n    \"2016-05-24 13:26 UTC\"\n\n    iex> format_datetime(~N[2000-01-01 23:00:07], \"Etc/UTC\")\n    \"2000-01-01 23:00 UTC\"\n\n    iex> format_datetime(~N[2000-01-01 23:00:07], \"Etc/UTC\", :date_time_seconds_timezone)\n    \"2000-01-01 23:00:07 UTC\"\n\n    iex> format_datetime(nil, \"Etc/UTC\")\n    \"\"\n\n    iex> format_datetime(\"\", \"Etc/UTC\")\n    \"\"","ref":"Pyro.Component.Helpers.html#format_datetime/3-examples"},{"type":"function","title":"Pyro.Component.Helpers.get_nested/3","doc":"Safely get nested values from maps or keyword lists that may be `nil` or an otherwise non-map value at any point. Great for accessing nested assigns in a template.","ref":"Pyro.Component.Helpers.html#get_nested/3"},{"type":"function","title":"Examples - Pyro.Component.Helpers.get_nested/3","doc":"iex> get_nested(nil, [:one, :two, :three])\n    nil\n\n    iex> get_nested(%{one: nil}, [:one, :two, :three])\n    nil\n\n    iex> get_nested(%{one: %{two: %{three: 3}}}, [:one, :two, :three])\n    3\n\n    iex> get_nested(%{one: %{two: [three: 3]}}, [:one, :two, :three])\n    3\n\n    iex> get_nested([one: :nope], [:one, :two, :three])\n    nil\n\n    iex> get_nested([one: :nope], [:one, :two, :three], :default)\n    :default","ref":"Pyro.Component.Helpers.html#get_nested/3-examples"},{"type":"function","title":"Pyro.Component.Helpers.gettext/1","doc":"","ref":"Pyro.Component.Helpers.html#gettext/1"},{"type":"function","title":"Pyro.Component.Helpers.local_now/0","doc":"","ref":"Pyro.Component.Helpers.html#local_now/0"},{"type":"function","title":"Pyro.Component.Helpers.local_now/1","doc":"","ref":"Pyro.Component.Helpers.html#local_now/1"},{"type":"function","title":"Pyro.Component.Helpers.toggle_class/3","doc":"A [`JS`](`Phoenix.LiveView.JS`) function to toggle CSS classes, since Phoenix does not yet provide one out of the box.","ref":"Pyro.Component.Helpers.html#toggle_class/3"},{"type":"function","title":"Examples - Pyro.Component.Helpers.toggle_class/3","doc":"toggle_class(\"rotate-180 bg-green\", to: \"#icon\")","ref":"Pyro.Component.Helpers.html#toggle_class/3-examples"},{"type":"function","title":"Pyro.Component.Helpers.translate_error/1","doc":"","ref":"Pyro.Component.Helpers.html#translate_error/1"},{"type":"function","title":"Pyro.Component.Helpers.translate_errors/2","doc":"Translates the errors for a field from a keyword list of errors.","ref":"Pyro.Component.Helpers.html#translate_errors/2"},{"type":"module","title":"Pyro.LiveComponent","doc":"This is basically a wrapper around `Phoenix.LiveComponent`, but it uses `Pyro.Component` instead of `Phoenix.Component` to enable Pyro's extended features.\n\n```\nuse Pyro.LiveComponent\n```\n\n> ##","ref":"Pyro.LiveComponent.html"},{"type":"module","title":"Note: {: .info} - Pyro.LiveComponent","doc":">\n> Please see the `Phoenix.LiveComponent` docs, as they will not be duplicated here.","ref":"Pyro.LiveComponent.html#module-note-info"},{"type":"module","title":"Pyro.LiveView","doc":"This is basically a wrapper around `Phoenix.LiveView`, but it uses `Pyro.Component` instead of `Phoenix.Component` to enable Pyro's extended features.\n\n```\nuse Pyro.LiveView\n```\n\n> ##","ref":"Pyro.LiveView.html"},{"type":"module","title":"Note: {: .info} - Pyro.LiveView","doc":">\n> Please see the `Phoenix.LiveView` docs, as they will not be duplicated here.","ref":"Pyro.LiveView.html#module-note-info"},{"type":"extras","title":"About","doc":"# About\n\nPyro is a suite of libraries for building UI in Phoenix.\n\n- [Pyro](https://hexdocs.pm/pyro)\n\n  Component tooling for Phoenix.\n\n  - Customizable \"overrides\" system for granularly customizable themes\n  - Extended component attributes, e.g. CSS merging\n\n- [PyroComponents](https://hexdocs.pm/pyro_components)\n\n  Ready-made Phoenix components, built with pyro.\n\n  - Heex component library\n  - Overrides presets to get started quickly while allowing deep customization\n\n- [AshPyro](https://hexdocs.pm/ash_pyro)\n\n  Declarative UI for Ash Framework.\n\n  - Ash extensions providing a declarative UI DSL\n\n- [AshPyroComponents](https://hexdocs.pm/ash_pyro_components)\n\n  Components that automatically render PyroComponents declaratively via AshPyro.\n\nTo install Pyro and write your own components, follow the [Get Started](get-started.html) guide. For the other features, please see the \"Get Started\" guide for the appropriate library instead.","ref":"about.html"},{"type":"extras","title":"What \"problem\" is it solving? - About","doc":"The default model of Phoenix is to generate, then customize. While this does provide great isolation, I have found it pretty tedious to repeatedly make very similar copy & paste boilerplate changes to the generated code every time I create a new Phoenix app. Additionally, some things (like timezone localization) are quite complicated and it would be nice for them to be handled by a library that is updated with future improvements. Copy & pasting boilerplate will lead to maintenance burdens down the road.\n\nThe tricky part is handling all the bespoke features in each app while sharing as much as possible. The goal is to provide a wide array of tooling, helpers and components with sane defaults, while allowing very granular overrides and optional libraries. By separating each level of features, you can choose which parts of Pyro to leverage, and which to implement yourself.","ref":"about.html#what-problem-is-it-solving"},{"type":"extras","title":"Get Started","doc":"# Get Started\n\nThis guide steps through the installation process for Pyro.","ref":"get-started.html"},{"type":"extras","title":"Installation - Get Started","doc":"The installation process is pretty straightforward.\n\n#","ref":"get-started.html#installation"},{"type":"extras","title":"Steps - Get Started","doc":"These steps assume you are adding Pyro to an existing Phoenix LiveView app, as generated from the most recent version of `phx.new`.\n\n1. Add `:pyro` to your dependencies:\n\n   ```elixir\n   def deps do\n     [\n      {:pyro, \"~> 0.3.2\"},\n\n      #","ref":"get-started.html#steps"},{"type":"extras","title":"OPTIONAL DEPS BELOW ### - Get Started","doc":"# Date/Time/Zone components & tooling\n      {:tz, \"~> 0.26\"},\n      {:tz_extra, \"~> 0.26\"},\n      #   or\n      {:tzdata, \"~> 1.1\"},\n     ]\n   end\n   ```\n\n2. Add `:pyro` to your `.formatter.exs`:\n\n   ```elixir\n   [\n     import_deps: [:ecto, :ecto_sql, :phoenix, :pyro],\n     subdirectories: [\"priv/*/migrations\"],\n     plugins: [Phoenix.LiveView.HTMLFormatter],\n     inputs: [\"*.{heex,ex,exs}\", \"{config,lib,test}/**/*.{heex,ex,exs}\", \"priv/*/seeds.exs\"]\n   ]\n   ```\n\n3. Add the following to your `config.exs`:\n\n   ```elixir\n   config :pyro, :overrides, [MyApp.Overrides]\n   config :pyro, gettext: MyApp.Gettext # optional\n   ```\n\n   See `Pyro.Overrides` to learn how to create your own overrides file.\n\n4. Edit your `my_app_web.ex` file, replacing:\n\n   - `Phoenix.Component` with `Pyro.Component`\n   - `Phoenix.LiveComponent` with `Pyro.LiveComponent`\n   - `Phoenix.LiveView` with `Pyro.LiveView`\n\n   **Note:** _Only_ replace those top-level modules, _do not_ replace submodules, e.g. `Phoenix.LiveView.Router`.\n\n5. (Optional) configure some runtime options in `runtime.exs`:\n\n   ```elixir\n   config :pyro, default_timezone: \"America/Chicago\"\n   ```","ref":"get-started.html#optional-deps-below"}],"content_type":"text/markdown"}